package skiplist.model;
import java.util.Optional;
import java.util.Stack;
import java.util.Iterator;

public class SkipList<E extends Comparable<E>> implements Iterable<E> {
	private SkipNode<E> front;
	private int size;

	public SkipList() {
		// Create a dummy SkipNode which will be 
		front = new SkipNode<E>(Optional.empty());
		front.addNext(Optional.empty());
		size = 0;
	}

	public int size() {
		return size;
	}
	
	public boolean isEmpty() {
		return (size == 0);
	}

	public int getHeight() {
		return front.getHeight();
	}

	/**
	 * Create a skip list node and insert it into the skip list. The stack of skip
	 * list nodes are the immediately previous nodes on each level (with level 0 on
	 * the top of the stack). At each level, set the next reference of the new node
	 * on that level to what the previous node on that level used to point to, and
	 * sets the previous node on that level to point to the new node. Assumes
	 * that the correct position has already been found, i.e. it just inserts the
	 * new node in the position given by the stack; it does not compare the data to
	 * anything to see whether it is in the right place.
	 * 
	 * @param data
	 *            The data to insert.
	 * @param prevs
	 *            The stack of immediately previous nodes on each level.
	 *            Precondition: the size of the stack is equal to the current height
	 *            of the skip list. In particular the stack is not empty, since we
	 *            at least need to know the previous node on level 0.
	 * @return The newly created node.
	 */
	private SkipNode<E> insertSkipListNode(E data, Stack<SkipNode<E>> prevs) {
	}

	/**
	 * Locate a search item, by returning a stack of nodes, one per level (level 0
	 * on the top of the stack). The node at level i either points to the node
	 * containing the search item, or to the first node on level i containing an
	 * item bigger than the search item (or null if there is no such bigger item on
	 * level i).
	 * 
	 * @param toFind
	 *            The search item.
	 * @return The stack of nodes immediately previous to the item on each level.
	 */
	public Stack<SkipNode<E>> locate(E toFind) {
	}

	@Override
	public String toString() {
		int maxLen = findMaxLength();

		String out = "";
		int ht = getHeight();
		Optional<SkipNode<E>> cur = Optional.of(front);
		while (cur.isPresent()) {
			Optional<E> curData = cur.get().getData();
			out += pad(maxLen, curData.isPresent() ? "" + curData.get() : "D") + " : ";
			for (int i = 0; i < cur.get().getHeight(); i++) {
				Optional<SkipNode<E>> n = cur.get().getNext(i);
				out += pad(maxLen, n.isPresent() && n.get().getData().isPresent() ? "" + n.get().getData().get() : "/") + " ";
			}
			for (int i = cur.get().getHeight(); i < ht; i++) {
				out += pad(maxLen, "|") + " ";
			}
			out += "\n";
			cur = cur.get().getNext(0);
		}

		return out;
	}
	
	private int findMaxLength() {
		int maxLen = 0;
		Optional<SkipNode<E>> cur = front.getNext(0);
		while (cur.isPresent()) {
			int l = cur.get().getData().get().toString().length();
			if (l > maxLen)
				maxLen = l;
			cur = cur.get().getNext(0);
		}
		return maxLen;
	}

	private static String pad(int n, String s) {
		while (s.length() < n) {
			s = " " + s;
		}
		return s;
	}

	@Override
	public Iterator<E> iterator() {
		return new SkipListIterator<>(front.getNext(0));
	}
}